/*

TEST OF CIPHERs USING VECTORS FROM CRYPTO++ v5.21 by Wei Dai

*/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include ".\..\..\..\include\cryptohash.h"
#pragma comment(linker,"/DEFAULTLIB:.\\..\\..\\..\\lib\\cryptohash.lib")

#define EncOK(x) \
if (x) {                        \
printf("ENCRYPTION SUCCESS\n"); \
} else {                        \
printf("ENCRYPTION FAILED\n");  \
getchar(); /*break if error*/   \
}

#define DecOK(x) \
if (x) {                        \
printf("DECRYPTION SUCCESS\n"); \
} else {                        \
printf("DECRYPTION FAILED\n");  \
getchar(); /*break if error*/   \
}

#define PrintHex(kk,ks,tmp) \
HexEncode(kk,ks,tmp);       \
printf(":%s\n",tmp);


int __stdcall GetStrings(FILE *f,BYTE *k,BYTE *p,BYTE *c,int *keylen)
{
	BYTE tmp[0x400]; char *v; int i,j,l; DWORD *kpc=&k;
	memset(&tmp,0,sizeof(tmp));
	j=fread(&tmp,sizeof(BYTE),sizeof(tmp),f);
	v=&tmp; *keylen=0;
	for ( i=0; ((i<3) && (j>0)); i++ ) {
		v=strtok(v," \r\n\t");
		l=strlen(v); j-=l;
		if (i==0) *keylen=l>>1;
		if (!l) return 0;
		HexDecode(v,*kpc++);
		v=(char *)((int)v+l+1); j--; if (j<=0) return 1;
		while (*v==' '||*v=='\t'||*v=='\n'||*v=='\r') { v++; j--; if (j<=0) return 1; }
	}
	if (j!=0) fseek(f,-j,SEEK_CUR);
	return 1;
}

VerifyRC5()
{
	BYTE key[16],plain[8],cipher[8],tmp[8]; DWORD i,keylen; BYTE sztmp[0x200];
	FILE *f=fopen("rc5val.dat","rb");
	if (f)
	{
		for ( i=1;;i++ )
		{
			if ((!GetStrings(f,&key,&plain,&cipher,&keylen))) break;
			printf("\n[RC5 TEST %i]\n",i);
			PrintHex(&key,keylen,&sztmp);
			PrintHex(&plain,sizeof(plain),&sztmp);
			RC5Init(&key);
			RC5Encrypt(&plain,&tmp);
			PrintHex(&tmp,sizeof(tmp),&sztmp);
			PrintHex(&cipher,sizeof(cipher),&sztmp);
			EncOK(memcmp(&tmp,&cipher,sizeof(tmp))==0);
			RC5Decrypt(&tmp,&tmp);
			PrintHex(&tmp,sizeof(tmp),&sztmp);
			DecOK(memcmp(&tmp,&plain,sizeof(tmp))==0);
			if (feof(f)) break;
		}
		fclose(f);
	}
	else printf("\n[RC5 file not found]\n");
}

VerifyRC6()
{
	BYTE key[32],plain[16],cipher[16],tmp[16]; DWORD i,keylen; BYTE sztmp[0x200];
	FILE *f=fopen("rc6val.dat","rb");
	if (f)
	{
		for ( i=1;;i++ )
		{
			if ((!GetStrings(f,&key,&plain,&cipher,&keylen))) break;
			printf("\n[RC6 TEST %i]\n",i);
			PrintHex(&key,keylen,&sztmp);
			PrintHex(&plain,sizeof(plain),&sztmp);
			RC6Init(&key,keylen);
			RC6Encrypt(&plain,&tmp);
			PrintHex(&tmp,sizeof(tmp),&sztmp);
			PrintHex(&cipher,sizeof(cipher),&sztmp);
			EncOK(memcmp(&tmp,&cipher,sizeof(tmp))==0);
			RC6Decrypt(&tmp,&tmp);
			PrintHex(&tmp,sizeof(tmp),&sztmp);
			DecOK(memcmp(&tmp,&plain,sizeof(tmp))==0);
			if (feof(f)) break;
		}
		fclose(f);
	}
	else printf("\n[RC6 file not found]\n");
}

Verify3Way()
{
	BYTE key[12],plain[12],cipher[12],tmp[12]; DWORD i,keylen; BYTE sztmp[0x200];
	FILE *f=fopen("3wayval.dat","rb");
	if (f)
	{
		for ( i=1;;i++ )
		{
			if ((!GetStrings(f,&key,&plain,&cipher,&keylen))) break;
			printf("\n[3-Way TEST %i]\n",i);
			PrintHex(&key,keylen,&sztmp);
			PrintHex(&plain,sizeof(plain),&sztmp);
			ThreeWayInit(&key);
			ThreeWayEncrypt(&plain,&tmp);
			PrintHex(&tmp,sizeof(tmp),&sztmp);
			PrintHex(&cipher,sizeof(cipher),&sztmp);
			EncOK(memcmp(&tmp,&cipher,sizeof(tmp))==0);
			ThreeWayDecrypt(&tmp,&tmp);
			PrintHex(&tmp,sizeof(tmp),&sztmp);
			DecOK(memcmp(&tmp,&plain,sizeof(tmp))==0);
			if (feof(f)) break;
		}
		fclose(f);
	}
	else printf("\n[ThreeWay file not found]\n");
}

VerifyCAST128()
{
	BYTE key[16],plain[8],cipher[8],tmp[8]; DWORD i,keylen; BYTE sztmp[0x200];
	FILE *f=fopen("cast128v.dat","rb");
	if (f)
	{
		for ( i=1;;i++ )
		{
			/* IMPORTANT FOR CAST-128!
			   key must be zeroed out if length is less than 16 bytes! 
			*/	memset(&key,0,sizeof(key));
			
			if ((!GetStrings(f,&key,&plain,&cipher,&keylen))) break;
			printf("\n[CAST-128 TEST %i]\n",i);
			PrintHex(&key,keylen,&sztmp);
			PrintHex(&plain,sizeof(plain),&sztmp);
			CAST128SetKey(&key,keylen);
			CAST128Encrypt(&plain,&tmp);
			PrintHex(&tmp,sizeof(tmp),&sztmp);
			PrintHex(&cipher,sizeof(cipher),&sztmp);
			EncOK(memcmp(&tmp,&cipher,sizeof(tmp))==0);
			CAST128Decrypt(&tmp,&tmp);
			PrintHex(&tmp,sizeof(tmp),&sztmp);
			DecOK(memcmp(&tmp,&plain,sizeof(tmp))==0);
			if (feof(f)) break;
		}
		fclose(f);
	}
	else printf("\n[CAST-128 file not found]\n");
}

VerifyGOST()
{
	BYTE key[32],plain[8],cipher[8],tmp[8]; DWORD i,keylen; BYTE sztmp[0x200];
	FILE *f=fopen("gostval.dat","rb");
	if (f)
	{
		for ( i=1;;i++ )
		{
			if ((!GetStrings(f,&key,&plain,&cipher,&keylen))) break;
			printf("\n[GOST TEST %i]\n",i);
			PrintHex(&key,keylen,&sztmp);
			PrintHex(&plain,sizeof(plain),&sztmp);
			GOSTSetKey(&key);
			GOSTEncrypt(&plain,&tmp);
			PrintHex(&tmp,sizeof(tmp),&sztmp);
			PrintHex(&cipher,sizeof(cipher),&sztmp);
			EncOK(memcmp(&tmp,&cipher,sizeof(tmp))==0);
			GOSTDecrypt(&tmp,&tmp);
			PrintHex(&tmp,sizeof(tmp),&sztmp);
			DecOK(memcmp(&tmp,&plain,sizeof(tmp))==0);
			if (feof(f)) break;
		}
		fclose(f);
	}
	else printf("\n[GOST file not found]\n");
}

VerifyIDEA()
{
	BYTE key[32],plain[8],cipher[8],tmp[8]; DWORD i,keylen; BYTE sztmp[0x200];
	FILE *f=fopen("ideaval.dat","rb");
	if (f)
	{
		for ( i=1;;i++ )
		{
			if ((!GetStrings(f,&key,&plain,&cipher,&keylen))) break;
			printf("\n[IDEA TEST %i]\n",i);
			PrintHex(&key,keylen,&sztmp);
			PrintHex(&plain,sizeof(plain),&sztmp);
			IDEASetKeyEnc(&key);
			IDEAEncrypt(&plain,&tmp);
			PrintHex(&tmp,sizeof(tmp),&sztmp);
			PrintHex(&cipher,sizeof(cipher),&sztmp);
			EncOK(memcmp(&tmp,&cipher,sizeof(tmp))==0);
			IDEASetKeyDec(&key);
			IDEADecrypt(&tmp,&tmp);
			PrintHex(&tmp,sizeof(tmp),&sztmp);
			DecOK(memcmp(&tmp,&plain,sizeof(tmp))==0);
			if (feof(f)) break;
		}
		fclose(f);
	}
	else printf("\n[IDEA file not found]\n");
}

VerifyMARS()
{
	BYTE key[32],plain[16],cipher[16],tmp[16]; DWORD i,keylen; BYTE sztmp[0x200];
	FILE *f=fopen("marsval.dat","rb");
	if (f)
	{
		for ( i=1;;i++ )
		{
			if ((!GetStrings(f,&key,&plain,&cipher,&keylen))) break;
			printf("\n[MARS TEST %i]\n",i);
			PrintHex(&key,keylen,&sztmp);
			PrintHex(&plain,sizeof(plain),&sztmp);
			MARSSetKey(&key,keylen);
			MARSEncrypt(&plain,&tmp);
			PrintHex(&tmp,sizeof(tmp),&sztmp);
			PrintHex(&cipher,sizeof(cipher),&sztmp);
			EncOK(memcmp(&tmp,&cipher,sizeof(tmp))==0);
			MARSDecrypt(&tmp,&tmp);
			PrintHex(&tmp,sizeof(tmp),&sztmp);
			DecOK(memcmp(&tmp,&plain,sizeof(tmp))==0);
			if (feof(f)) break;
		}
		fclose(f);
	}
	else printf("\n[MARS file not found]\n");
}

VerifyDES()
{
	BYTE key[8],plain[8],cipher[8],tmp[8]; DWORD i,keylen; BYTE sztmp[0x200];
	FILE *f=fopen("descert.dat","rb");
	if (f)
	{
		for ( i=1;;i++ )
		{
			if ((!GetStrings(f,&key,&plain,&cipher,&keylen))) break;
			printf("\n[DES TEST %i]\n",i);
			PrintHex(&key,keylen,&sztmp);
			PrintHex(&plain,sizeof(plain),&sztmp);
			DESSetKeyEnc(&key);
			DESEncrypt(&plain,&tmp);
			PrintHex(&tmp,sizeof(tmp),&sztmp);
			PrintHex(&cipher,sizeof(cipher),&sztmp);
			EncOK(memcmp(&tmp,&cipher,sizeof(tmp))==0);
			DESSetKeyDec(&key);
			DESDecrypt(&tmp,&tmp);
			PrintHex(&tmp,sizeof(tmp),&sztmp);
			DecOK(memcmp(&tmp,&plain,sizeof(tmp))==0);
			if (feof(f)) break;
		}
		fclose(f);
	}
	else printf("\n[DES file not found]\n");
}

VerifyTwofish()
{
	BYTE key[32],plain[16],cipher[16],tmp[16]; DWORD i,keylen; BYTE sztmp[0x200];
	FILE *f=fopen("twofishv.dat","rb");
	if (f)
	{
		for ( i=1;;i++ )
		{
			if ((!GetStrings(f,&key,&plain,&cipher,&keylen))) break;
			printf("\n[Twofish TEST %i]\n",i);
			PrintHex(&key,keylen,&sztmp);
			PrintHex(&plain,sizeof(plain),&sztmp);
			TwofishInit(&key,keylen);
			TwofishEncrypt(&plain,&tmp);
			PrintHex(&tmp,sizeof(tmp),&sztmp);
			PrintHex(&cipher,sizeof(cipher),&sztmp);
			EncOK(memcmp(&tmp,&cipher,sizeof(tmp))==0);
			TwofishDecrypt(&tmp,&tmp);
			PrintHex(&tmp,sizeof(tmp),&sztmp);
			DecOK(memcmp(&tmp,&plain,sizeof(tmp))==0);
			if (feof(f)) break;
		}
		fclose(f);
	}
	else printf("\n[Twofish file not found]\n");
}


VerifyRijndael()
{
	BYTE key[32],plain[16],cipher[16],tmp[16]; DWORD i,keylen; BYTE sztmp[0x200];
	FILE *f=fopen("rijndael.dat","rb");
	if (f)
	{
		for ( i=1;;i++ )
		{
			if ((!GetStrings(f,&key,&plain,&cipher,&keylen))) break;
			printf("\n[Rijndael TEST %i]\n",i);
			PrintHex(&key,keylen,&sztmp);
			PrintHex(&plain,sizeof(plain),&sztmp);
			RijndaelInit(&key,keylen);
			RijndaelEncrypt(&plain,&tmp);
			PrintHex(&tmp,sizeof(tmp),&sztmp);
			PrintHex(&cipher,sizeof(cipher),&sztmp);
			EncOK(memcmp(&tmp,&cipher,sizeof(tmp))==0);
			RijndaelDecrypt(&tmp,&tmp);
			PrintHex(&tmp,sizeof(tmp),&sztmp);
			DecOK(memcmp(&tmp,&plain,sizeof(tmp))==0);
			if (feof(f)) break;
		}
		fclose(f);
	}
	else printf("\n[rijndael file not found]\n");
}

int main(int argc,char *argv[])
{
	printf("\n[CIPHER TEST USING VECTORS FROM CRYPTO++ v5.21 by Wei Dai]\n");
	VerifyRC5();
	VerifyRC6();
	Verify3Way();
	VerifyCAST128();
	VerifyGOST();
	VerifyIDEA();
	VerifyMARS();
	VerifyDES();
	VerifyTwofish();
	VerifyRijndael();
	return;
}

